@@ -1,5 +1,7 @@
 pub mod score;
 
+use typed_generational_arena::StandardArena;
+
 use crate::{
     index::*,
     query::score::calculator::ScoreCalculator,
@@ -9,11 +11,10 @@ use std::{
     collections::{HashMap, HashSet},
     fmt::Debug,
     hash::Hash,
-    sync::{Arc, Mutex},
 };
 
 use self::score::calculator::{FieldData, TermData};
-
+extern crate typed_generational_arena;
 /**
  * Query Result.
 
@@ -65,7 +66,7 @@ Arguments
 returns Array of QueryResult structs
 */
 pub fn query<T: Eq + Hash + Clone + Debug, M, S: ScoreCalculator<T, M>>(
-    index: &mut Index<T>,
+    mut index: &mut Index<T>,
     query: &str,
     score_calculator: &mut S,
     tokenizer: Tokenizer,
@@ -73,53 +74,22 @@ pub fn query<T: Eq + Hash + Clone + Debug, M, S: ScoreCalculator<T, M>>(
     fields_boost: &[f64],
     removed: Option<&HashSet<T>>,
 ) -> Vec<QueryResult<T>> {
-    let docs = &index.docs;
-    let fields = &index.fields;
-    let query_terms = tokenizer(&query);
+    let query_terms = tokenizer(query);
     let mut scores: HashMap<T, f64> = HashMap::new();
     for query_term_pre_filter in &query_terms {
-        let query_term = filter(&query_term_pre_filter);
+        let query_term = filter(query_term_pre_filter);
+        print!("{}", query_term);
         if !query_term.is_empty() {
-            let expanded_terms = expand_term(&index, &query_term);
+            let expanded_terms = expand_term(index, &query_term, &index.arena_index);
             let mut visited_documents_for_term: HashSet<T> = HashSet::new();
             for query_term_expanded in expanded_terms {
                 let term_node_option =
-                    find_inverted_index_node(Arc::clone(&index.root), &query_term_expanded);
-                if let Some(term_node) = term_node_option {
-                    let mut term_node_borrowed = term_node.lock().unwrap();
-                    let mut new_first_doc = None;
-                    let mut assign_new_first_doc = false;
-                    let mut document_frequency = 0;
-
-                    if let Some(term_node_option_first_doc) = &mut term_node_borrowed.first_doc {
-                        let mut prev_pointer: Option<Arc<Mutex<DocumentPointer<T>>>> = None;
-                        let mut pointer_option = Some(Arc::clone(&term_node_option_first_doc));
-                        while let Some(pointer) = pointer_option {
-                            if removed.is_some() // Cleanup old removed documents while searching. If vaccume after delete, this will have not effect
-                                && removed
-                                    .unwrap()
-                                    .contains(&pointer.lock().unwrap().details.lock().unwrap().key)
-                            {
-                                if let Some(pp) = &prev_pointer {
-                                    pp.lock().unwrap().next = pointer.lock().unwrap().next.clone();
-                                } else {
-                                    new_first_doc = (&pointer.lock().unwrap().next).clone();
-                                    assign_new_first_doc = true;
-                                    //  term_node_borrowed.first_doc = (&pointer.borrow().next).clone();
-                                }
-                            } else {
-                                prev_pointer = Some(Arc::clone(&pointer));
-                                document_frequency += 1;
-                            }
-                            pointer_option = pointer.lock().unwrap().next.clone();
-                        }
-                    }
-
-                    if assign_new_first_doc {
-                        term_node_borrowed.first_doc = new_first_doc;
-                    }
-
-                    if let Some(term_node_option_first_doc) = &mut term_node_borrowed.first_doc {
+                    find_inverted_index_node(index.root, &query_term_expanded, &index.arena_index);
+                if let Some(term_node_index) = term_node_option {
+                    let document_frequency =
+                        disconnect_and_count_documents(&mut index, term_node_index, removed);
+                    let term_node = index.arena_index.get(term_node_index).unwrap();
+                    if let Some(term_node_option_first_doc) = term_node.first_doc {
                         if document_frequency > 0 {
                             let term_expansion_data = TermData {
                                 all_query_terms: &query_terms,
@@ -129,19 +99,19 @@ pub fn query<T: Eq + Hash + Clone + Debug, M, S: ScoreCalculator<T, M>>(
                             let pre_calculations = &score_calculator.before_each(
                                 &term_expansion_data,
                                 document_frequency,
-                                docs,
+                                &index.docs,
                             );
 
-                            let mut pointer = Some(Arc::clone(&term_node_option_first_doc));
+                            let mut pointer = Some(term_node_option_first_doc);
                             while let Some(p) = pointer {
-                                let pointer_borrowed = p.lock().unwrap();
-                                let details = &pointer_borrowed.details.lock().unwrap();
-                                let key = &details.key;
-                                if removed.is_none() || !removed.unwrap().contains(&key) {
+                                let pointer_borrowed = index.arena_doc.get(p).unwrap();
+                                let key = &pointer_borrowed.details_key;
+                                if removed.is_none() || !removed.unwrap().contains(key) {
+                                    let fields = &index.fields;
                                     let score = &score_calculator.score(
                                         pre_calculations.as_ref(),
-                                        &pointer_borrowed,
-                                        &details,
+                                        pointer_borrowed,
+                                        index.docs.get(key).unwrap(),
                                         &FieldData {
                                             fields_boost,
                                             fields,
@@ -151,14 +121,14 @@ pub fn query<T: Eq + Hash + Clone + Debug, M, S: ScoreCalculator<T, M>>(
                                     if let Some(s) = score {
                                         let new_score = max_score_merger(
                                             s,
-                                            scores.get(&key),
-                                            visited_documents_for_term.contains(&key),
+                                            scores.get(key),
+                                            visited_documents_for_term.contains(key),
                                         );
                                         scores.insert(key.to_owned(), new_score);
                                         visited_documents_for_term.insert(key.to_owned());
                                     }
                                 }
-                                pointer = pointer_borrowed.next.clone();
+                                pointer = pointer_borrowed.next;
                             }
                         }
                     }
@@ -185,11 +155,20 @@ Expands term with all possible combinations.
  * `term` Term.
 returns All terms that starts with `term` string.
  */
-fn expand_term<I: Debug>(index: &Index<I>, term: &str) -> Vec<String> {
-    let node = find_inverted_index_node(Arc::clone(&index.root), term);
+fn expand_term<I: Debug>(
+    index: &Index<I>,
+    term: &str,
+    arena_index: &StandardArena<InvertedIndexNode<I>>,
+) -> Vec<String> {
+    let node = find_inverted_index_node(index.root, term, &index.arena_index);
     let mut results = Vec::new();
     if let Some(n) = node {
-        expand_term_from_node(n, &mut results, term);
+        expand_term_from_node(
+            index.arena_index.get(n).unwrap(),
+            &mut results,
+            term,
+            arena_index,
+        );
     }
 
     results
@@ -204,23 +183,21 @@ Recursively goes through inverted index nodes and expands term with all possible
  * `term Term.
  */
 fn expand_term_from_node<I: Debug>(
-    node: Arc<Mutex<InvertedIndexNode<I>>>,
+    node: &InvertedIndexNode<I>,
     results: &mut Vec<String>,
     term: &str,
+    arena_index: &StandardArena<InvertedIndexNode<I>>,
 ) {
-    if node.lock().unwrap().first_doc.is_some() {
+    if node.first_doc.is_some() {
         results.push(term.to_owned());
     }
-    let nl = node.lock().unwrap();
-    let mut child = nl.first_child.clone();
-    std::mem::drop(nl);
-    while let Some(c) = child {
-        let cb = c.lock().unwrap();
+    let mut child = node.first_child;
+    while let Some(child_index) = child {
+        let cb = arena_index.get(child_index).unwrap();
         let mut inter = term.to_owned();
         inter.push(cb.char);
-        std::mem::drop(cb);
-        expand_term_from_node(Arc::clone(&c), results, &inter); // String.fromCharCode(child.charCode)
-        child = c.lock().unwrap().next.clone();
+        expand_term_from_node(cb, results, &inter, arena_index); // String.fromCharCode(child.charCode)
+        child = cb.next;
     }
 }
 
@@ -230,11 +207,7 @@ mod tests {
     fn approx_equal(a: f64, b: f64, dp: u8) -> bool {
         let p: f64 = 10f64.powf(-(dp as f64));
 
-        if (a - b).abs() < p {
-            return true;
-        } else {
-            return false;
-        }
+        (a - b).abs() < p
     }
 
     use super::*;
@@ -255,7 +228,7 @@ mod tests {
         Some(d.text.as_str())
     }
 
-    fn filter(s: &String) -> String {
+    fn filter(s: &str) -> String {
         s.to_owned()
     }
 
@@ -264,7 +237,7 @@ mod tests {
 
         #[test]
         fn it_should_return_doc_1() {
-            let mut idx: Index<usize> = create_index(2);
+            let mut index = create_index::<usize>(2);
             let docs = vec![
                 Doc {
                     id: 1,
@@ -279,7 +252,7 @@ mod tests {
             ];
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -288,12 +261,12 @@ mod tests {
                 );
             }
             let result = query(
-                &mut idx,
+                &mut index,
                 &"a".to_string(),
                 &mut crate::query::score::default::bm25::new(),
                 tokenizer,
                 filter,
-                &vec![1., 1.],
+                &[1., 1.],
                 None,
             );
             assert_eq!(result.len(), 1);
@@ -306,7 +279,7 @@ mod tests {
 
         #[test]
         fn it_should_return_doc_1_and_2() {
-            let mut idx: Index<usize> = create_index(2);
+            let mut index = create_index::<usize>(2);
             let docs = vec![
                 Doc {
                     id: 1,
@@ -322,7 +295,7 @@ mod tests {
 
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -332,12 +305,12 @@ mod tests {
             }
 
             let result = query(
-                &mut idx,
+                &mut index,
                 &"c".to_string(),
                 &mut crate::query::score::default::bm25::new(),
                 tokenizer,
                 filter,
-                &vec![1., 1.],
+                &[1., 1.],
                 None,
             );
             assert_eq!(result.len(), 2);
@@ -362,7 +335,7 @@ mod tests {
 
         #[test]
         fn it_should_match_text_by_expanding() {
-            let mut idx: Index<usize> = create_index(2);
+            let mut index = create_index::<usize>(2);
             let docs = vec![
                 Doc {
                     id: 1,
@@ -378,7 +351,7 @@ mod tests {
 
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -388,12 +361,12 @@ mod tests {
             }
 
             let result = query(
-                &mut idx,
+                &mut index,
                 &"h".to_string(),
                 &mut crate::query::score::default::bm25::new(),
                 tokenizer,
                 filter,
-                &vec![1., 1.],
+                &[1., 1.],
                 None,
             );
             assert_eq!(result.len(), 1);
@@ -406,7 +379,7 @@ mod tests {
 
         #[test]
         fn it_should_use_filter_for_query() {
-            let mut idx: Index<usize> = create_index(2);
+            let mut index = create_index::<usize>(2);
             let docs = vec![
                 Doc {
                     id: 1,
@@ -422,7 +395,7 @@ mod tests {
 
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -431,19 +404,19 @@ mod tests {
                 );
             }
 
-            fn custom_filter(s: &String) -> String {
-                if s.as_str() == "a" {
+            fn custom_filter(s: &str) -> String {
+                if s == "a" {
                     return "".to_string();
                 }
                 filter(s)
             }
             let result = query(
-                &mut idx,
+                &mut index,
                 &"a".to_string(),
                 &mut crate::query::score::default::bm25::new(),
                 tokenizer,
                 custom_filter,
-                &vec![1., 1.],
+                &[1., 1.],
                 None,
             );
             assert_eq!(result.len(), 0);
@@ -451,7 +424,7 @@ mod tests {
 
         #[test]
         fn it_should_use_token_separator_as_disjunction_operator() {
-            let mut idx: Index<usize> = create_index(2);
+            let mut index = create_index::<usize>(2);
             let docs = vec![
                 Doc {
                     id: 1,
@@ -467,7 +440,7 @@ mod tests {
 
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -477,12 +450,12 @@ mod tests {
             }
 
             let result = query(
-                &mut idx,
+                &mut index,
                 &"a d".to_string(),
                 &mut crate::query::score::default::bm25::new(),
                 tokenizer,
                 filter,
-                &vec![1., 1.],
+                &[1., 1.],
                 None,
             );
             assert_eq!(result.len(), 2);
@@ -510,7 +483,7 @@ mod tests {
 
         #[test]
         fn it_should_expand_all() {
-            let mut idx: Index<usize> = create_index(1);
+            let mut index = create_index::<usize>(2);
             let docs: Vec<Doc> = vec![
                 Doc {
                     id: 1,
@@ -526,7 +499,7 @@ mod tests {
 
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -534,13 +507,13 @@ mod tests {
                     doc,
                 );
             }
-            let exp = expand_term(&idx, &"a".to_string());
+            let exp = expand_term(&index, &"a".to_string(), &index.arena_index);
             assert_eq!(exp, vec!["adef".to_string(), "abc".to_string()]);
         }
 
         #[test]
         fn it_should_not_expand() {
-            let mut idx: Index<usize> = create_index(2);
+            let mut index = create_index::<usize>(2);
             let docs = vec![
                 Doc {
                     id: 1,
@@ -556,7 +529,7 @@ mod tests {
 
             for doc in docs {
                 add_document_to_index(
-                    &mut idx,
+                    &mut index,
                     &[title_extract, text_extract],
                     tokenizer,
                     filter,
@@ -564,7 +537,7 @@ mod tests {
                     doc,
                 );
             }
-            let exp = expand_term(&idx, &"x".to_string());
+            let exp = expand_term(&index, &"x".to_string(), &index.arena_index);
             assert_eq!(exp, Vec::new() as Vec<String>);
         }
     }
